// Copyright ® 2024 OneSpan North America, Inc. All rights reserved. 

 
/////////////////////////////////////////////////////////////////////////////
//
//
// This file is example source code. It is provided for your information and
// assistance. See your licence agreement for details and the terms and
// conditions of the licence which governs the use of the source code. By using
// such source code you will be accepting these terms and conditions. If you do
// not wish to accept these terms and conditions, DO NOT OPEN THE FILE OR USE
// THE SOURCE CODE.
//
// Note that there is NO WARRANTY.
//
//////////////////////////////////////////////////////////////////////////////


import CoreImage
import CoreMedia
import Combine

class CombineCameraModel: ObservableObject {
    @Published var frame: CGImage?
    @Published var error: Error?

    private let cameraManager = CameraManager()
    private var cancellables = Set<AnyCancellable>()

    var scannerDelegate: CustomScanningDelegate?

    private let context = CIContext()
    
    init() {
        setupSubscriptions()
    }
    
    deinit {
        cancellables.forEach { $0.cancel() }
    }
    
    func setupSubscriptions() {
        // frame update subscription
        cameraManager.$current
            .receive(on: RunLoop.main)
            .compactMap { [weak self] buffer -> (CGImage?, CMSampleBuffer)? in
                guard let buffer = buffer else {
                    return nil
                }
                
                guard let image = CGImage.create(from: buffer.imageBuffer) else {
                    return nil
                }

                let ciImage = CIImage(cgImage: image)
                
                DispatchQueue.global().async {
                    self?.scannerDelegate?.didCaptureBuffer(buffer)
                }
                            
                return (self?.context.createCGImage(ciImage, from: ciImage.extent), buffer)
            }
            .map{ (image: CGImage?, _) in
                return image
            }
            .sink { [weak self] image in
                self?.frame = image
            }
            .store(in: &cancellables)
        
        // error subscription
        cameraManager.$error
            .receive(on: RunLoop.main)
            .map{ $0 }
            .sink { [weak self] error in
                self?.error = error
            }
            .store(in: &cancellables)
    }
    
    // MARK: - Public methods
    func stopSession() {
        cameraManager.stopSession()
    }
    
    func startSession() {
        cameraManager.startSession()
    }
}
