// Copyright ® 2024 OneSpan North America, Inc. All rights reserved. 

 
/////////////////////////////////////////////////////////////////////////////
//
//
// This file is example source code. It is provided for your information and
// assistance. See your licence agreement for details and the terms and
// conditions of the licence which governs the use of the source code. By using
// such source code you will be accepting these terms and conditions. If you do
// not wish to accept these terms and conditions, DO NOT OPEN THE FILE OR USE
// THE SOURCE CODE.
//
// Note that there is NO WARRANTY.
//
//////////////////////////////////////////////////////////////////////////////


import SwiftUI
import MSSImageScanner
import CoreMedia

/// The sample presents 3 different ways to integrate QRCodeScannerSDK
struct SampleMain: View {
    // Title labels
    private let titleAppName      = "Image Scanner SDK Sample"
    private let titleImageContent = "Image content :"
    private let titleImageType    = "Image type :"

    // Scan result labels
    @State public var scanResultLabel = ""
    @State public var scanResultType  = ""
    
    // Button titles
    private let buttonTitleSystemCamera  = "Launch System Camera"
    private let buttonTitleCustomScanner = "Launch Custom Scanner"
    private let buttonTitleModernScanner = "Launch Combine Scanner"
    
    // This sample presents usage of a system camera to take a single photo to decode
    @State private var isShowingSystemCamera = false
    @State private var systemCameraDelegate: ScanResultDelegate?
    
    // This sample presents usage of a custom scanner view and `decodeImage` SDK method
    @State private var isShowingCustomScanner = false
    @State private var customScannerResultsDelegate: ScanResultDelegate?
    
    // This sample represents usage of a SwiftUI views based scanner with Combine elements, the video buffer is processed by using `decodeImage`
    @State private var isShowingCombineScanner = false
    @State private var combineScannerResultsDelegate: ScanResultDelegate?
    
    @State private var hasAppeared = false
    
    // Dark mode support
    @Environment(\.colorScheme) var colorScheme
    var backgroundColor: Color {
        colorScheme == .dark ? .black : .white
    }
    
    var body: some View {
        ZStack {
            VStack(spacing: 0) {
                headerView
                
                scannedText
                scannedType

                Spacer()
                
                buttonsView
            }
            .frame(maxHeight: .infinity)
            .background(backgroundColor)
        }
        .onAppear{
            setupView()
        }
        .fullScreenCoverCustom(isPresented: $isShowingSystemCamera) {
            SystemCameraImageDecodingSample(resultDelegate: $systemCameraDelegate,
                                            isPresented: $isShowingSystemCamera)
        }
        .fullScreenCoverCustom(isPresented: $isShowingCustomScanner) {
            CustomScanner(resultDelegate: $customScannerResultsDelegate,
                          isPresented: $isShowingCustomScanner)
        }
        .fullScreenCoverCustom(isPresented: $isShowingCombineScanner) {
            CombineCameraView(resultDelegate: $combineScannerResultsDelegate,
                              isPresented: $isShowingCombineScanner)
        }
    }
    
    private func setupView() {
        if !hasAppeared {
            systemCameraDelegate = CameraResultDelegate(parent: self)
            
            customScannerResultsDelegate = CameraResultDelegate(parent: self)
            
            combineScannerResultsDelegate = CameraResultDelegate(parent: self)
            hasAppeared = true
        }
    }
}

// Header view
extension SampleMain {
    var headerView: some View {
        HeaderView(title: titleAppName)
    }
}

// Description Views
extension SampleMain {
    var scannedText: some View {
        DescriptionView(title: titleImageContent,
                        description: scanResultLabel,
                        multilineDescription: true)
    }
    
    var scannedType: some View {
        DescriptionView(title: titleImageType,
                        description: scanResultType)
    }
}

// Action Buttons
extension SampleMain {
    var buttonsView: some View {
        VStack {
            Button(buttonTitleSystemCamera) {
                launchSystemCamera()
            }
            .padding(20)

            Button(buttonTitleCustomScanner) {
                launchCustomScanner()
            }
            .padding(20)

            Button(buttonTitleModernScanner) {
                launchCombineScanner()
            }
            .padding(20)
        }
    }
    
    private func launchSystemCamera() {
        isShowingSystemCamera = true
    }

    private func launchCustomScanner() {
        isShowingCustomScanner = true
    }
    
    private func launchCombineScanner() {
        isShowingCombineScanner = true
    }
}

struct SampleMainView_Previews: PreviewProvider {
    static var previews: some View {
        SampleMain()
    }
}
